// Timer.cpp: implementation of the CTimer class.
//
//////////////////////////////////////////////////////////////////////

#include "Timer.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTimer::CTimer()
{
	//
	// Initialize some default values 
	//
	m_CPUTicksPerMicroSecond = 1;

	//
	// We need to determine the clock speed of the processor.  First make sure
	// the processor supports the RDTSC Function.
	//
	__try
	{
		_asm
		{
			RDTSC
		}
	}
	__except(1)
	{
		//
		// RDTSC not supported.  Not sure what to do here!
		//
		return;
	}

	//
	// Now, calculate the actual number of CPU Ticks per MicroSecond.  We could use any
	// number of methods here, the preferred being QueryPerformanceCounter, but we're going 
	// to use the multimediaa clock because under Windows XP QueryPerformanceCounter may not be
	// valid.
	//

	// This loop waits for the next tick
	// so that we begin speed testing on a tick edge
	int timeStart, timeStop;
	__int64 t1, t2, diff, elapsedTime;
	timeStart = timeGetTime();
	for(;;)
	{
		timeStop = timeGetTime();
		if (  (int)(timeStop - timeStart) > 0 )
		{
			t1 = GetCPUTicks();
			break;	
		}	
	}

	timeStart = timeStop;

	for(;;)
	{
		timeStop = timeGetTime();
		if (  (int)(timeStop - timeStart) > 100 )
		{
			t2 = GetCPUTicks();
			break;	
		}	
	}

	diff = t2 - t1;
	elapsedTime = (__int64)(timeStop - timeStart);

	m_CPUTicksPerMicroSecond = (diff) / (elapsedTime * 1000);
}


CTimer::~CTimer()
{

}


// Get the current value of the clockticks counter...
__int64 CTimer::GetCPUTicks()
{
	BigInt CPUTicks;

	_asm
	{
		RDTSC
		mov CPUTicks.i32[0], eax
		mov CPUTicks.i32[4], edx
	}

	return CPUTicks.i64;
}

// Get the current time (according to clockticks)...
__int64 CTimer::GetTime()
{
	BigInt CPUTicks;

	_asm
	{
		RDTSC
		mov CPUTicks.i32[0], eax
        mov CPUTicks.i32[4], edx
	}

	return CPUTicks.i64 / m_CPUTicksPerMicroSecond;
}
