//-----------------------------------------------------------------------------
// File: skybox.h
//
// Desc: Encapsulation of skybox geometry and textures
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#ifndef _SKYBOX_H
#define _SKYBOX_H


struct SKYBOX_VERTEX
{
    D3DXVECTOR3 pos;
    D3DXVECTOR2 tex;

    static DWORD FVF;
};



class CSkybox
{
public:
    CSkybox();

    HRESULT Create( LPDIRECT3DDEVICE9 pDevice, float fSize = 1.0f );
    HRESULT Destroy();

    HRESULT InitDeviceObjects();
    HRESULT RestoreDeviceObjects();
    HRESULT Render();
    HRESULT InvalidateDeviceObjects();
    HRESULT DeleteDeviceObjects();

    HRESULT CreateTexturesFromCross( LPDIRECT3DTEXTURE9 pTexCross );
    LPDIRECT3DCUBETEXTURE9 GetEnvMap() { return m_pTexCubeEnvMap; }
    
protected:
    HRESULT CreateTextureFromRect( LPDIRECT3DTEXTURE9 pTexSrc, LPDIRECT3DTEXTURE9* ppTexDest, RECT& rc );
    HRESULT FillEnvMap();
    HRESULT RotateSurface180( LPDIRECT3DSURFACE9 pSurface );

protected:
    LPDIRECT3DTEXTURE9 m_pTexPosX;
    LPDIRECT3DTEXTURE9 m_pTexNegX;
    LPDIRECT3DTEXTURE9 m_pTexPosY;
    LPDIRECT3DTEXTURE9 m_pTexNegY;
    LPDIRECT3DTEXTURE9 m_pTexPosZ;
    LPDIRECT3DTEXTURE9 m_pTexNegZ;

    LPDIRECT3DCUBETEXTURE9 m_pTexCubeEnvMap;

    LPDIRECT3DVERTEXBUFFER9 m_pVB;
    LPDIRECT3DDEVICE9 m_pd3dDevice;
    
    float m_fSize;


};

#endif //_SKYBOX_H_